/* main module, the only hardware dependent one */

#include <stdint.h>
#include <unistd.h>	/* usleep */
#include "basic.h"
#include "calc.h"
#include "common.h"
#include "conio.h"
#include "main.h"

extern uint8_t ram[RAMSIZE];
extern unsigned int pc;
extern unsigned int datasp;


int main (void)
{
  FILE *fp;
  int i;

  if ((fp = fopen ("ram.bin", "rb")) != NULL)
  {
    (void) fread (ram, sizeof (uint8_t), (size_t) RAMSIZE, fp);
    (void) fclose (fp);
  }

  printf ("\033[2J");
  reset ();
/* 0B9C: */
  ram[0x8265u-RAMSTART] = 0;
/* 0BA0: */
  ram[0x8264u-RAMSTART] &= 0x20;
  while (1) {
    do {
      ram[0x8258u-RAMSTART] = 0;
      ram[0x8259u-RAMSTART] = 0;
      i = do_direct ();
    } while (i >= 0 || i == DONE);
    error_handler (i);
  }
/* while (read16 (0x826B) != 1234); /* wyjscie awaryjne: wiersz o numerze 1234 */

  return 0;
}


int key1;


/* roughly 0144: stops the CPU then waits for a key to be pressed */
void key_wait (void)
{
  key1 = getch ();
}


/* roughly 0820 to 08DA: returns the scan code of the pressed key, or 0 if the
 key is not pressed or not recognized */
int key_scan (void)
{
  const int key_conversion_tab[128] = {
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x04, 0x00, 0x00, 0x04, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x0A, 0x00, 0x00, 0x00, 0x00, 0x00,	/* ^Z -> 0x0A, init */
    0x15, 0x02, 0x00, 0x07, 0x08, 0x00, 0x1B, 0x00,
    0x00, 0x00, 0x19, 0x18, 0x00, 0x17, 0x16, 0x1A,
    0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x10, 0x11, 0x12,
    0x13, 0x14, 0x00, 0x00, 0x05, 0x1C, 0x06, 0x00,
    0x03, 0x1D, 0x1E, 0x1F, 0x20, 0x21, 0x22, 0x23,
    0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2A, 0x2B,
    0x2C, 0x2D, 0x2E, 0x2F, 0x30, 0x31, 0x32, 0x33,
    0x34, 0x35, 0x36, 0x00, 0x00, 0x00, 0x09, 0x00,
    0x00, 0x1D, 0x1E, 0x1F, 0x20, 0x21, 0x22, 0x23,
    0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2A, 0x2B,
    0x2C, 0x2D, 0x2E, 0x2F, 0x30, 0x31, 0x32, 0x33,
    0x34, 0x35, 0x36, 0x0A, 0x04, 0x00, 0x01, 0x00
  };
  int x;
  x = (key1 != 0) ? key1 : (kbhit () != 0) ? getch () : 0;
  key1 = 0;
  x %= 128;
  if (x != 0)
  {
    printf ("\033[18;1H%d  ", x);
  }
  return key_conversion_tab[x];
}


/* was the Esc=STOP key pressed? */
int stop_scan (void)
{
  int x;
  x = (key1 != 0) ? key1 : (kbhit () != 0) ? getch () : 0;
  if (x == 0x1B)
  {
    key1 = 0;
    return 1;
  }
  return 0;
}


/*
    .    .    .    .    .    .    .    .    .    .
     S  RUN                      _   _   _   _
EXT          DEG  RAD  GRA  TR  |_| |_| |_| |_|  STOP
     F  WRT                     |_| |_| |_| |_|
*/

void lcd (unsigned int addr, uint8_t data)
{
  unsigned int x;

  switch (addr)
  {
    case 0x80:
      printf (((data & 0x01) == 0) ? "\033[2;1H   " : "\033[2;1HEXT");
      printf (((data & 0x02) == 0) ? "\033[1;6H " : "\033[1;6HS");
      printf (((data & 0x04) == 0) ? "\033[3;6H " : "\033[3;6HF");
      break;
    case 0x88:
      printf (((data & 0x01) == 0) ? "\033[1;9H   " : "\033[1;9HRUN");
      printf (((data & 0x02) == 0) ? "\033[3;9H   " : "\033[3;9HWRT");
      printf (((data & 0x10) == 0) ? "\033[2;14H   " : "\033[2;14HDEG");
      break;
    case 0x90:
      break;
    case 0x98:
      printf (((data & 0x01) == 0) ? "\033[2;19H   " : "\033[2;19HRAD");
      break;
    case 0xA0:
      printf (((data & 0x01) == 0) ? "\033[2;24H   " : "\033[2;24HGRA");
      break;
    case 0xA8:
      printf (((data & 0x01) == 0) ? "\033[2;29H  " : "\033[2;29HTR");
      break;
    case 0xB0:
      printf (((data & 0x01) == 0) ? "\033[2;33H " : "\033[2;33H|");
      printf (((data & 0x02) == 0) ? "\033[2;34H " : "\033[2;34H_");
      printf (((data & 0x04) == 0) ? "\033[3;33H " : "\033[3;33H|");
      printf (((data & 0x08) == 0) ? "\033[3;34H " : "\033[3;34H_");
      printf (((data & 0x10) == 0) ? "\033[3;35H " : "\033[3;35H|");
      break;
    case 0xB8:
      printf (((data & 0x01) == 0) ? "\033[2;35H " : "\033[2;35H|");
      printf (((data & 0x02) == 0) ? "\033[1;34H " : "\033[1;34H_");
      printf (((data & 0x04) == 0) ? "\033[2;37H " : "\033[2;37H|");
      printf (((data & 0x08) == 0) ? "\033[2;38H " : "\033[2;38H_");
      printf (((data & 0x10) == 0) ? "\033[3;37H " : "\033[3;37H|");
      break;
    case 0xC0:
      printf (((data & 0x01) == 0) ? "\033[3;38H " : "\033[3;38H_");
      printf (((data & 0x02) == 0) ? "\033[3;39H " : "\033[3;39H|");
      printf (((data & 0x04) == 0) ? "\033[2;39H " : "\033[2;39H|");
      printf (((data & 0x08) == 0) ? "\033[1;38H " : "\033[1;38H_");
      printf (((data & 0x10) == 0) ? "\033[2;41H " : "\033[2;41H|");
      break;
    case 0xC8:
      printf (((data & 0x01) == 0) ? "\033[2;42H " : "\033[2;42H_");
      printf (((data & 0x02) == 0) ? "\033[3;41H " : "\033[3;41H|");
      printf (((data & 0x04) == 0) ? "\033[3;42H " : "\033[3;42H_");
      printf (((data & 0x08) == 0) ? "\033[3;43H " : "\033[3;43H|");
      printf (((data & 0x10) == 0) ? "\033[2;43H " : "\033[2;43H|");
      break;
    case 0xD0:
      printf (((data & 0x01) == 0) ? "\033[1;42H " : "\033[1;42H_");
      printf (((data & 0x02) == 0) ? "\033[2;45H " : "\033[2;45H|");
      printf (((data & 0x04) == 0) ? "\033[2;46H " : "\033[2;46H_");
      printf (((data & 0x08) == 0) ? "\033[3;45H " : "\033[3;45H|");
      printf (((data & 0x10) == 0) ? "\033[3;46H " : "\033[3;46H_");
      break;
    case 0xD8:
      printf (((data & 0x01) == 0) ? "\033[3;47H " : "\033[3;47H|");
      printf (((data & 0x02) == 0) ? "\033[2;47H " : "\033[2;47H|");
      printf (((data & 0x04) == 0) ? "\033[1;46H " : "\033[1;46H_");
      printf (((data & 0x08) == 0) ? "\033[2;50H    " : "\033[2;50HSTOP");
      break;
    case 0xE0:
      printf ("\033[12;1H");
      for (x=0; x<SCREEN_WIDTH; ++x)
      {
        printf (((data & 0x0F) != x) ? "      " :
		((data & 0x10) == 0) ? "+++++ " : "----- ");
      }
      break;
    case 0xE8:
      break;
    default:
      printf ("\033[%u;%uH", (addr & 7) + 4, ((addr / 8) & 0x0F) * 6 + 1);
      for (x=0; x<5; ++x)
      {
        printf(((data & 1) == 0) ? "\033[m " : "\033[7m ");
        data >>= 1;
      }
      printf("\033[m ");
      break;
  }
}

void delay_ms (unsigned int x)
{
  usleep (1000 * x);
}
